@@ -39,6 +39,7 @@ from redux.basic_types import (
     FinishAction,
     FinishEvent,
     InitAction,
+    MethodSelf,
     ReducerType,
     ReturnType,
     SelectorOutput,
@@ -55,6 +56,7 @@ from redux.basic_types import (
     is_state_reducer_result,
 )
 from redux.serialization_mixin import SerializationMixin
+from redux.utils import call_func, signature_without_selector
 
 if TYPE_CHECKING:
     from collections.abc import Callable
@@ -71,7 +73,6 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         """Create a new store."""
         self.store_options = options or StoreOptions()
         self.reducer = reducer
-        self._create_task = self.store_options.task_creator
 
         self._action_middlewares = list(self.store_options.action_middlewares)
         self._event_middlewares = list(self.store_options.event_middlewares)
@@ -95,7 +96,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         self._workers = [
             self.store_options.side_effect_runner_class(
                 task_queue=self._event_handlers_queue,
-                create_task=self._create_task,
+                create_task=self.store_options.task_creator,
             )
             for _ in range(self.store_options.side_effect_threads)
         ]
@@ -120,12 +121,17 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         for listener_ in self._listeners.copy():
             if isinstance(listener_, weakref.ref):
                 listener = listener_()
-                assert listener is not None  # noqa: S101
+                if listener is None:
+                    msg = (
+                        'Listener has been garbage collected. '
+                        'Consider using `keep_ref=True` if it suits your use case.'
+                    )
+                    raise RuntimeError(msg)
             else:
                 listener = listener_
             result = listener(state)
-            if asyncio.iscoroutine(result) and self._create_task:
-                self._create_task(result)
+            if asyncio.iscoroutine(result) and self.store_options.task_creator:
+                self.store_options.task_creator(result)
 
     def _run_actions(self: Store[State, Action, Event]) -> None:
         while len(self._actions) > 0:
@@ -309,7 +315,7 @@ class Store(Generic[State, Action, Event], SerializationMixin):
                 Concatenate[SelectorOutput, Args],
                 AwaitableOrNot[ReturnType],
             ],
-        ) -> AutorunReturnType[ReturnType, Args]:
+        ) -> AutorunReturnType[Args, ReturnType]:
             return self.store_options.autorun_class(
                 store=self,
                 selector=selector,
@@ -333,6 +339,10 @@ class Store(Generic[State, Action, Event], SerializationMixin):
             func: Callable[
                 Concatenate[SelectorOutput, Args],
                 ReturnType,
+            ]
+            | Callable[
+                Concatenate[MethodSelf, SelectorOutput, Args],
+                ReturnType,
             ],
         ) -> ViewReturnType[ReturnType, Args]: ...
         @overload
@@ -340,13 +350,20 @@ class Store(Generic[State, Action, Event], SerializationMixin):
             func: Callable[
                 Concatenate[SelectorOutput, Args],
                 Awaitable[ReturnType],
+            ]
+            | Callable[
+                Concatenate[MethodSelf, SelectorOutput, Args],
+                Awaitable[ReturnType],
             ],
         ) -> ViewReturnType[Awaitable[ReturnType], Args]: ...
-
         def view_decorator(
             func: Callable[
                 Concatenate[SelectorOutput, Args],
                 AwaitableOrNot[ReturnType],
+            ]
+            | Callable[
+                Concatenate[MethodSelf, SelectorOutput, Args],
+                AwaitableOrNot[ReturnType],
             ],
         ) -> ViewReturnType[AwaitableOrNot[ReturnType], Args]:
             _options = options or ViewOptions()
@@ -372,6 +389,8 @@ class Store(Generic[State, Action, Event], SerializationMixin):
     def with_state(
         self: Store[State, Action, Event],
         selector: Callable[[State], SelectorOutput],
+        *,
+        ignore_uninitialized_store: bool = False,
     ) -> WithStateDecorator[SelectorOutput]:
         """Wrap a function, passing the result of the selector as its first argument.
 
@@ -383,30 +402,44 @@ class Store(Generic[State, Action, Event], SerializationMixin):
         `store._state` is also possible.
         """
 
+        @overload
         def with_state_decorator(
             func: Callable[
                 Concatenate[SelectorOutput, Args],
                 ReturnType,
             ],
-        ) -> Callable[Args, ReturnType]:
+        ) -> Callable[Args, ReturnType]: ...
+        @overload
+        def with_state_decorator(
+            func: Callable[
+                Concatenate[MethodSelf, SelectorOutput, Args],
+                ReturnType,
+            ],
+        ) -> Callable[Concatenate[MethodSelf, Args], ReturnType]: ...
+        def with_state_decorator(
+            func: Callable[
+                Concatenate[SelectorOutput, Args],
+                ReturnType,
+            ]
+            | Callable[
+                Concatenate[MethodSelf, SelectorOutput, Args],
+                ReturnType,
+            ],
+        ) -> (
+            Callable[Args, ReturnType]
+            | Callable[Concatenate[MethodSelf, Args], ReturnType]
+        ):
             def wrapper(*args: Args.args, **kwargs: Args.kwargs) -> ReturnType:
                 if self._state is None:
+                    if ignore_uninitialized_store:
+                        return cast('ReturnType', None)
                     msg = 'Store has not been initialized yet.'
                     raise RuntimeError(msg)
-                return func(selector(self._state), *args, **kwargs)
+                return call_func(func, [selector(self._state)], *args, **kwargs)
 
-            signature = inspect.signature(func)
-            parameters = list(signature.parameters.values())
-            if parameters and parameters[0].kind in [
-                inspect.Parameter.POSITIONAL_ONLY,
-                inspect.Parameter.POSITIONAL_OR_KEYWORD,
-            ]:
-                parameters = parameters[1:]
-
-            wrapped = wraps(func)(wrapper)
-            wrapped.__signature__ = signature.replace(  # pyright: ignore [reportAttributeAccessIssue]
-                parameters=parameters,
-            )
+            signature = signature_without_selector(func)
+            wrapped = wraps(cast('Any', func))(wrapper)
+            wrapped.__signature__ = signature  # pyright: ignore [reportAttributeAccessIssue]
 
             return wrapped
 
