@@ -17,7 +17,7 @@ from redux.basic_types import FinishEvent, State
 if TYPE_CHECKING:
     from collections.abc import Callable
 
-    from _pytest.fixtures import SubRequest
+    from _pytest.fixtures import SubRequest  # pyright: ignore[reportPrivateImportUsage]
 
     from redux.main import Store
 
@@ -162,7 +162,7 @@ def store_snapshot(
         request.config.getoption(
             '--override-store-snapshots',
             default=cast(
-                Any,
+                'Any',
                 str_to_bool(os.environ.get('REDUX_TEST_OVERRIDE_SNAPSHOTS', 'false'))
                 == 1,
             ),
